namespace Syntriax.Engine.Core;

/// <summary>
/// Indicates the object is an <see cref="IAssignable"/> with an assignable <see cref="IUniverse"/> field.
/// </summary>
public interface IHasUniverse : IAssignable
{
    /// <summary>
    /// Event triggered when the <see cref="IUniverse"/> value has has been assigned a new value.
    /// </summary>
    event UniverseAssignedEventHandler? OnUniverseAssigned;

    /// <inheritdoc cref="IUniverse" />
    IUniverse Universe { get; }

    /// <summary>
    /// Assign a value to the <see cref="IUniverse"/> field of this object.
    /// </summary>
    /// <param name="universe">New <see cref="IUniverse"/> to assign.</param>
    /// <returns> 
    /// <see cref="true"/>, if the value given assigned successfully assigned, <see cref="false"/> if not.
    /// </returns>
    bool Assign(IUniverse universe);

    delegate void UniverseAssignedEventHandler(IHasUniverse sender);
}
