namespace Syntriax.Engine.Core;

/// <summary>
/// Represents a basic entity in the engine.
/// </summary>
public interface IEntity : IInitializable, IHasStateEnable
{
    /// <summary>
    /// Event triggered when the <see cref="Id"/> of the <see cref="IEntity"/> changes.
    /// The string action parameter is the previous <see cref="Id"/> of the <see cref="IEntity"/>. 
    /// </summary>
    event IdChangedEventHandler? OnIdChanged;

    /// <summary>
    /// The ID of the <see cref="IEntity"/>.
    /// </summary>
    string Id { get; set; }

    delegate void IdChangedEventHandler(IEntity sender, string previousId);
}
