using System.Collections;
using System.Collections.Generic;

namespace Syntriax.Engine.Core;

public class CoroutineManager : UniverseObject
{
    private readonly List<IEnumerator> enumerators = [];

    public IEnumerator StartCoroutine(IEnumerator enumerator)
    {
        enumerators.Add(enumerator);
        return enumerator;
    }

    public void StopCoroutine(IEnumerator enumerator)
    {
        enumerators.Remove(enumerator);
    }

    protected override void OnEnteringUniverse(IUniverse universe)
    {
        universe.OnUpdate += OnUpdate;
    }

    protected override void OnExitingUniverse(IUniverse universe)
    {
        universe.OnUpdate -= OnUpdate;
    }

    private void OnUpdate(IUniverse sender, UniverseTime time)
    {
        for (int i = enumerators.Count - 1; i >= 0; i--)
        {
            if (enumerators[i].Current is ICoroutineYield coroutineYield && coroutineYield.Yield())
                continue;

            if (!enumerators[i].MoveNext())
                enumerators.RemoveAt(i);
        }
    }
}
