using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

namespace Syntriax.Engine.Core;

public static class BehaviourExtensions
{
    public static T? FindBehaviour<T>(this IEnumerable<IUniverseObject> universeObjects) where T : class
    {
        foreach (IUniverseObject universeObject in universeObjects)
            if (universeObject.BehaviourController.GetBehaviour<T>() is T behaviour)
                return behaviour;

        return default;
    }

    public static bool TryFindBehaviour<T>(this IEnumerable<IUniverseObject> universeObjects, [NotNullWhen(returnValue: true)] out T? behaviour) where T : class
    {
        behaviour = FindBehaviour<T>(universeObjects);
        return behaviour is not null;
    }

    public static void FindBehaviours<T>(this IEnumerable<IUniverseObject> universeObjects, List<T> behaviours) where T : class
    {
        behaviours.Clear();
        List<T> cache = [];

        foreach (IUniverseObject universeObject in universeObjects)
        {
            universeObject.BehaviourController.GetBehaviours(cache);
            behaviours.AddRange(cache);
        }
    }
}
