using System.Collections;
using System.Collections.Generic;

using Syntriax.Engine.Core;

namespace Syntriax.Engine.Physics2D;

public class PhysicsCoroutineManager : UniverseObject
{
    private readonly List<IEnumerator> enumerators = [];
    private IPhysicsEngine2D? physicsEngine = null;

    public IEnumerator CreateCoroutine(IEnumerator enumerator)
    {
        enumerators.Add(enumerator);
        return enumerator;
    }

    public void StopCoroutine(IEnumerator enumerator)
    {
        enumerators.Remove(enumerator);
    }

    protected override void OnEnteringUniverse(IUniverse universe)
    {
        physicsEngine = universe.GetUniverseObject<IPhysicsEngine2D>();
        if (physicsEngine is IPhysicsEngine2D foundPhysicsEngine)
            foundPhysicsEngine.OnPhysicsStep += OnPhysicsStep;
        else
            universe.OnUpdate += OnUpdate;
    }

    private void OnPhysicsStep(IPhysicsEngine2D sender, float stepDeltaTime)
    {
        for (int i = enumerators.Count - 1; i >= 0; i--)
        {
            if (enumerators[i].Current is ICoroutineYield coroutineYield && coroutineYield.Yield())
                continue;

            if (!enumerators[i].MoveNext())
                enumerators.RemoveAt(i);
        }
    }

    protected override void OnExitingUniverse(IUniverse universe)
    {
        if (physicsEngine is IPhysicsEngine2D existingPhysicsEngine)
            existingPhysicsEngine.OnPhysicsStep -= OnPhysicsStep;
        universe.OnUpdate -= OnUpdate;
    }

    private void OnUpdate(IUniverse sender, UniverseTime engineTime)
    {
        if (Universe is not IUniverse universe)
            return;

        physicsEngine = universe.GetUniverseObject<IPhysicsEngine2D>();
        if (physicsEngine is IPhysicsEngine2D foundPhysicsEngine)
        {
            foundPhysicsEngine.OnPhysicsStep += OnPhysicsStep;
            universe.OnUpdate -= OnUpdate;
        }
    }
}
