using System;

using Syntriax.Engine.Core;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class CircleConverter : EngineTypeYamlSerializerBase<Circle>
{
    public override Circle Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        parser.Consume<MappingStart>();

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(Circle.Center)) != 0)
            throw new ArgumentException($"{nameof(Circle)} mapping must start with {nameof(Circle.Center)}");
        Vector2D lowerBoundary = (Vector2D)rootDeserializer(typeof(Vector2D))!;

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(Circle.Radius)) != 0)
            throw new ArgumentException($"{nameof(Circle)} mapping must end with {nameof(Circle.Radius)}");
        float radius = (float)rootDeserializer(typeof(float))!;

        parser.Consume<MappingEnd>();

        return new Circle(lowerBoundary, radius);
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        Circle circle = (Circle)value!;

        emitter.Emit(new MappingStart());
        emitter.Emit(new Scalar(nameof(Circle.Center)));
        serializer(circle.Center, typeof(Vector2D));
        emitter.Emit(new Scalar(nameof(Circle.Radius)));
        serializer(circle.Radius, typeof(float));
        emitter.Emit(new MappingEnd());
    }
}
