using System;

using Syntriax.Engine.Core;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class ColorHSVConverter : EngineTypeYamlSerializerBase<ColorHSV>
{
    private static readonly int SUBSTRING_START_LENGTH = nameof(ColorHSV).Length + 1;

    public override ColorHSV Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        string value = parser.Consume<Scalar>().Value;
        string insideParenthesis = value[SUBSTRING_START_LENGTH..^1];
        string[] values = insideParenthesis.Split(", ");
        return new ColorHSV(float.Parse(values[0]), float.Parse(values[1]), float.Parse(values[2]));
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        ColorHSV hsv = (ColorHSV)value!;
        emitter.Emit(new Scalar($"{nameof(ColorHSV)}({hsv.Hue}, {hsv.Saturation}, {hsv.Value})"));
    }
}
