using System;

using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Serialization;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class StateEnableConverter : EngineTypeYamlSerializerBase<IStateEnable>
{
    public override IStateEnable? Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        bool isTrackingController = ProgressionTracker.Progression.ApproximatelyEquals(0f);
        ProgressionTracker.Set(isTrackingController ? .25f : ProgressionTracker.Progression, $"Reading state enable");

        bool enabled;

        IStateEnable stateEnable;

        parser.Consume<MappingStart>();

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(IStateEnable.Enabled)) != 0)
            throw new();
        enabled = bool.Parse(parser.Consume<Scalar>().Value);

        if (parser.Consume<Scalar>().Value.CompareTo(SERIALIZED_SCALAR_NAME) != 0)
            throw new();
        SerializedClass instanceSerializedClass = (SerializedClass)rootDeserializer(typeof(SerializedClass))!;
        stateEnable = (IStateEnable)instanceSerializedClass.CreateInstance(EntityRegistry);

        parser.Consume<MappingEnd>();

        stateEnable.Enabled = enabled;

        ProgressionTracker.Set(isTrackingController ? .25f : ProgressionTracker.Progression, $"Read state enable");
        return stateEnable;
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        IStateEnable stateEnable = (IStateEnable)value!;

        bool isTrackingController = ProgressionTracker.Progression.ApproximatelyEquals(0f);
        ProgressionTracker.Set(isTrackingController ? .25f : ProgressionTracker.Progression, $"Serializing state enable");

        emitter.Emit(new MappingStart());

        emitter.Emit(new Scalar(nameof(IStateEnable.Enabled)));
        emitter.Emit(new Scalar(stateEnable.Enabled.ToString()));

        emitter.Emit(new Scalar(SERIALIZED_SCALAR_NAME));
        serializer(new SerializedClass(stateEnable));

        ProgressionTracker.Set(isTrackingController ? 1f : ProgressionTracker.Progression, $"Serialized state enable");
        emitter.Emit(new MappingEnd());
    }
}
